unit Volumen;
{***********************************************
 *            COMPONENTE TVOLUMEN v 1.3        *
 ***********************************************
 Versin: 1.3
 Autor: Delphius
 E-mail: HM_CARL@hotmail.com
 Web: *** En veremos ***
 Fecha: 05/01/05 - 16:34 pm
 ***********************************************
 Descripcin:
   TVolumen es un componente capz de cambiar el
 volumen del S.O mediante una simple API llama-
 da WaveOutSetVolume.
 ***********************************************
 Funcionamiento:
   El volumen del S.O se maneja en forma hexade-
 cimal, donde 0000 es el silencio y FFFF es el
 mximo para cada parlante.
   Para ello, TVolumen tiene una propiedad funda-
 mental: Multiplo. La cual slo puede tener valo-
 res dentro del rango [0,20], y es gracias a ella
 que se puede obtener el porcentaje del volumen
 como as tambin el volumen mismo.
 ***********************************************
 Correcciones:
   Beta
   Se ha agregado una variable fAntVolume que al-
 macena el volumen anterior, lo que permite res-
 taurar el volumen anterior.
   1.0
   Se ha agregado el constructor, el destructor y
 dos procedimientos: GetActualVolume y GetInitial
 Volume.
   1.1
   Se ha solucionado el problema del balanceo, en
 versiones anteriores el canal derecho se silen-
 ciaba.
   1.2
   Se ha eliminado la posibilidad de "resetar" el
 volumen anterior, ya que esto no funcionaba co-
 correctamente.
 **********************************************
 Agradecimientos:
   Este componente fue hecho gracias a la ayu-
 da proporcionada por la API-Guide y la ayuda
 brindada por Windows SDK y las refinaciones
 extradas de Trucomania.
   Gracias a mi hermano quien detect la falla
 en el parlante derecho...
 **********************************************
 Derechos de Autor:
 Este Componente es de libre uso. Cualquier
 modificacin que se desee hacer, notificar a:
 HM_CARL@hotmail.com
 **********************************************}
interface

uses
  Windows, Messages, SysUtils, Classes;

type
  { TRango delimita los valores permitidos }
  TRango = 0..20;
  { *** Definicin del Componente *** }
  TVolumen = class(TComponent)
  private
    fMultiplo: TRango;
    fPorc: integer;   // Vol. en porcentaje
    fVolumen: string;  // Vol. en Hexadecimal
    fSilencio: boolean; // Esta el silencio?
    { funciones }
    function GetPorcVolume: integer;
    function GetVolume: string;
    function GetSilencio: boolean;
    procedure GetInitialVolume;
  protected
  public
  { *** Propiedades pblicas (Solo Lectura) *** }
    property PorcVolume: integer read GetPorcVolume;
    property Volume: string read GetVolume;
    property Silencio: boolean read GetSilencio;
  { *** Procedimientos *** }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;
    procedure SetVolumen(NuevoMultiplo: TRango);
    procedure SetSilencio;
    procedure SubirVolumen;
    procedure BajarVolumen;
    procedure GetActualVolume;
  published
  { *** Propiedades publicadas *** }
    property Multiplo: TRango read fMultiplo write fMultiplo;
  end;

procedure Register;

implementation

uses MMsystem;

Function TVolumen.GetPorcVolume: integer;
{ Devolver el volumen en porcentaje }
begin
Result := fPorc;
end;

Function TVolumen.GetVolume: string;
{ Devolver el volumen en hexadecimal }
begin
Result := fVolumen;
end;

Function TVolumen.GetSilencio: boolean;
{ Devolver True en caso de que Vol = 00000000 }
begin
Result := fSilencio;
end;

Procedure TVolumen.SetVolumen(NuevoMultiplo: TRango);
{ Dar el volumen segn el rango }
var Vol:DWORD;
    Izq,Der: word;
begin
// Cambiar los valores "Multiplos"
fMultiplo := NuevoMultiplo;
Vol := 0;
// ** Parlante Izquierdo
Izq := Round(fMultiplo * 3276.75);
// ** Parlante Derecho
Der := Izq;
// Calcular el volumen
Vol := Vol or Izq;
Vol := Vol + (Der Shl 16);
fVolumen := IntToHex(Vol,4); { Hexadecimal }
fPorc:= (fMultiplo * 100) div 20; { porcentual }
// Subir volumen
WaveOutSetVolume(WAVE_MAPPER,Vol);
// Chequear silencio
if fMultiplo = 0
   then fSilencio := True
   else fSilencio := False;
end;

procedure TVolumen.SetSilencio;
{ Bajar el volumen del todo }
begin
SetVolumen(0);
end;

procedure TVolumen.SubirVolumen;
{ Subir el volumen en un rango }
begin
if fMultiplo < 20 then
SetVolumen(fMultiplo+1);
end;

procedure TVolumen.BajarVolumen;
{ Bajar el volumen en un rango }
begin
if fMultiplo > 0 then
SetVolumen(fMultiplo-1);
end;

procedure TVolumen.GetInitialVolume;
{ Obtener el volumen inicial del S.O }
var Vol:DWORD;
    aux:integer;
    Izq,Der : Word;
begin
// Obtener volumen
WaveOutGetVolume(WAVE_MAPPER,@Vol);
// ** Parlante Izquierdo
Izq := Vol and $FFFF;
// ** Parlante Derecho
Der := Vol Shr 16;
// Calcular...
fVolumen := IntToHex(Izq,4); { Hexadecimal }
aux := Round(Izq/3276.14);
fMultiplo := aux;
fPorc := (fMultiplo * 100) div 20; { Porcentual }
// Chequear Silencio
if fMultiplo = 0
   then fSilencio := True
   else fSilencio := False;
end;

Constructor TVOlumen.Create(AOwner: TComponent);
{ constructor }
begin
inherited Create(AOwner);
GetInitialVolume;
end;

Destructor TVolumen.Destroy;
{ Destructor }
begin
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Mi Paleta', [TVolumen]);
end;

procedure TVolumen.GetActualVolume;
{ Obtener el volumen actual del S.O }
begin
GetInitialVolume;
end;

end.
